export * from '@volar/language-service';
export * from '@volar/language-service/lib/utils/featureWorkers';
import type { Requests } from '@vue/typescript-plugin/lib/requests';
export declare function createVueLanguageServicePlugins(ts: typeof import('typescript'), client?: Requests): import("@volar/language-service").LanguageServicePlugin<any>[];
