import { type VueVirtualCode } from '@vue/language-core';
import type * as ts from 'typescript';
export declare function getComponentNames(ts: typeof import('typescript'), program: ts.Program, { fileName, sfc }: VueVirtualCode): string[];
