"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.create = create;
console.warn('[volar-service-pug] this module is not yet supported for web.');
function create() {
    return {
        name: 'pug (stub)',
        capabilities: {},
        create() {
            return {};
        },
    };
}
//# sourceMappingURL=empty.js.map