"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getComponentMeta = getComponentMeta;
const componentMeta_1 = require("vue-component-meta/lib/componentMeta");
const utils_1 = require("./utils");
function getComponentMeta(ts, program, language, getSourceScript, sourceFile, virtualCode, tag) {
    const checker = program.getTypeChecker();
    const componentType = (0, utils_1.getComponentType)(ts, checker, sourceFile, virtualCode, tag);
    if (!componentType) {
        return;
    }
    return (0, componentMeta_1.getComponentMeta)(ts, checker, ts.createPrinter(), language, getSourceScript, componentType.node, componentType.type, false);
}
//# sourceMappingURL=getComponentMeta.js.map