"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getLanguageService = getLanguageService;
const pugDocument_1 = require("./pugDocument");
const completion_1 = require("./services/completion");
const documentHighlight_1 = require("./services/documentHighlight");
const documentLinks_1 = require("./services/documentLinks");
const foldingRanges_1 = require("./services/foldingRanges");
const hover_1 = require("./services/hover");
const quoteComplete_1 = require("./services/quoteComplete");
const scanner_1 = require("./services/scanner");
const selectionRanges_1 = require("./services/selectionRanges");
function getLanguageService(htmlLs) {
    return {
        parsePugDocument: (0, pugDocument_1.register)(htmlLs),
        doComplete: (0, completion_1.register)(htmlLs),
        findDocumentHighlights: (0, documentHighlight_1.register)(htmlLs),
        findDocumentLinks: (0, documentLinks_1.register)(htmlLs),
        doHover: (0, hover_1.register)(htmlLs),
        createScanner: (0, scanner_1.register)(htmlLs),
        getSelectionRanges: (0, selectionRanges_1.register)(htmlLs),
        doQuoteComplete: (0, quoteComplete_1.register)(htmlLs),
        getFoldingRanges: (0, foldingRanges_1.register)(),
    };
}
//# sourceMappingURL=languageService.js.map