import type { DocumentSelector, FormattingOptions, LanguageServiceContext, LanguageServicePlugin, ProviderResult, TextDocument } from '@volar/language-service';
export declare function create({ documentSelector, isFormattingEnabled, getFormattingOptions, }?: {
    documentSelector?: DocumentSelector;
    isFormattingEnabled?(document: TextDocument, context: LanguageServiceContext): ProviderResult<boolean>;
    getFormattingOptions?(document: TextDocument, options: FormattingOptions, context: LanguageServiceContext): ProviderResult<{}>;
}): LanguageServicePlugin;
//# sourceMappingURL=index.d.ts.map