import { type VueVirtualCode } from '@vue/language-core';
import type * as ts from 'typescript';
export declare function getComponentSlots(ts: typeof import('typescript'), program: ts.Program, virtualCode: VueVirtualCode): string[];
