export declare const ctx = "__VLS_ctx";
export declare const dollars = "__VLS_dollars";
export declare const slots = "__VLS_slots";
export declare const props = "__VLS_props";
export declare const defaults = "__VLS_defaults";
export declare const defaultModels = "__VLS_defaultModels";
export declare const modelEmit = "__VLS_modelEmit";
export declare const emit = "__VLS_emit";
export declare const exposed = "__VLS_exposed";
export declare const setup = "__VLS_setup";
export declare const components = "__VLS_components";
export declare const directives = "__VLS_directives";
export declare const intrinsics = "__VLS_intrinsics";
export declare const _export = "__VLS_export";
export declare const ModelProps = "__VLS_ModelProps";
export declare const Props = "__VLS_Props";
export declare const Slots = "__VLS_Slots";
export declare const InheritedAttrs = "__VLS_InheritedAttrs";
export declare const TemplateRefs = "__VLS_TemplateRefs";
export declare const RootEl = "__VLS_RootEl";
export declare const ModelEmit = "__VLS_ModelEmit";
export declare const EmitProps = "__VLS_EmitProps";
export declare const Emit = "__VLS_Emit";
export declare const SetupExposed = "__VLS_SetupExposed";
export declare const PublicProps = "__VLS_PublicProps";
export declare const StyleModules = "__VLS_StyleModules";
