import type * as ts from 'typescript';
export declare function getElementAttrs(ts: typeof import('typescript'), program: ts.Program, fileName: string, tag: string): {
    name: string;
    type: string;
}[];
