import { type HTMLFormatConfiguration, Range, type TextDocument, type TextEdit } from 'vscode-html-languageservice';
export declare function format(document: TextDocument, range: Range | undefined, options: HTMLFormatConfiguration, voidElements?: string[]): TextEdit[];
