import { Segment } from './types';
export declare function clone(segments: Segment<any>[]): Segment<any>[];
export declare function generateMap(segments: Segment<any>[], readSource: (source?: string) => [number, string]): string;
